#ifndef OBJFN_H_
#define OBJFN_H_

/* exported object- and object-management-functions */

extern unsigned int objnew_AUDIO(void *);
extern unsigned int objnew_PLAYER(void *);
extern unsigned int objnew_BG(void *);
extern unsigned int objnew_BOUNDARY(void *, int, int);
extern unsigned int objnew_OHARD(void *, int, int);
extern unsigned int objnew_OSOFT(void *, int, int, const char *, const char *);
extern unsigned int objnew_HUNTER(void *);
extern unsigned int objnew_BOMB(void *, unsigned int, int, int);
extern unsigned int objnew_EXIT(void *);
extern unsigned int objnew_GD_DYN(void *, int, int);
extern unsigned int objnew_GD_HEART(void *, int, int);
extern unsigned int objnew_GD_KEY(void *, int, int);
extern unsigned int objnew_GD_KEYHOLE(void *, int, int);
extern unsigned int objnew_GD_BOMBS(void *, int, int);
extern unsigned int objnew_GD_PILL(void *, int, int);
extern unsigned int objnew_GD_PORTAL(void *, int, int);
extern unsigned int objnew_GD_GOLD(void *, int, int);

extern VG_BOOL objmgmt_HUNTER(void *);

#endif /* OBJFN_H_ */
